<?php
add_action('admin_init', 'inti_gath_settings_init');
add_action('admin_menu', 'inti_gath_settings_page');

define('INTI_GATH_MODULES', [
    'attributes' => 'Attributi (svojstva)',
    'categories' => 'Kategorije proizvoda',
    'images' => 'Slike',
    'image_upload' => 'Slike (Upload iz Gatha)',
    'prices' => 'Cijene proizvoda',
    'products' => 'Proizvodi',
    'stock' => 'Stanje zalihe',
    'tires' => 'Gume',
]);

/**
 * Inicijalizira postavke.
 */
function inti_gath_settings_init() {
    add_option('gath_base_uri');
    add_option('gath_webshop_type', 'retail');
    add_option('gath_motorcycles', false);
    add_option('gath_login_for_prices', false);
    add_option('gath_backorders', false);


    register_setting('gath_options', 'gath_base_uri', [
        'type' => 'string',
        'show_in_rest' => false,
    ]);

    register_setting('gath_options', 'gath_webshop_type', [
        'type' => 'string',
        'show_in_rest' => false,
    ]);

    register_setting('gath_options', 'gath_motorcycles', [
        'type' => 'boolean',
        'show_in_rest' => false,
    ]);

    register_setting('gath_options', 'gath_login_for_prices', [
        'type' => 'boolean',
        'show_in_rest' => false,
    ]);

    register_setting('gath_options', 'gath_backorders', [
        'type' => 'boolean',
        'show_in_rest' => false,
    ]);

    foreach (array_keys(INTI_GATH_MODULES) as $key) {
        add_option("gath_module_{$key}", false);
    }

    foreach (array_keys(INTI_GATH_MODULES) as $key) {
        register_setting('gath_options', "gath_module_{$key}", [
            'type' => 'boolean',
            'show_in_rest' => false,
        ]);
    }
}

/**
 * Dodaje stranicu s postavkama u izbornik.
 */
function inti_gath_settings_page() {
    add_options_page(
        'Inti Gath',
        'Inti Gath',
        'manage_options',
        'inti-gath',
        'inti_gath_settings_page_render'
    );
}

/**
 * Renderira stranicu s postavkama.
 */
function inti_gath_settings_page_render() {

    if (!current_user_can('manage_options')) {
        return;
    }

    if (isset($_GET['settings-updated'])) {
        add_settings_error( 'gath_messages', 'gath_message', 'Postavke spremljene', 'updated' );
    }

    $webshopType = get_option('gath_webshop_type');
    $motorcycles = get_option('gath_motorcycles');
    ?>
    <div class="wrap">
        <h1><?= esc_html(get_admin_page_title()); ?></h1>

        <form action="options.php" method="post">
            <table class="form-table">
                <tr>
                    <th scope="row"><label for="gath_base_uri">URI Gath servisa</label></th>
                    <td><input type="text" class="regular-text" id="gath_base_uri" name="gath_base_uri" value="<?= get_option('gath_base_uri'); ?>"></td>
                </tr>

                <tr>
                    <th scope="row"><label for="gath_webshop_type">Vrsta webshopa</label></th>
                    <td>
                        <select id="gath_webshop_type" name="gath_webshop_type">
                            <option value="retail"<?= $webshopType === 'retail' ? ' selected' : ''; ?>>Maloprodajni</option>
                            <option value="wholesale"<?= $webshopType === 'wholesale' ? ' selected' : ''; ?>>Veleprodajni</option>
                        </select>
                    </td>
                </tr>

                <tr>
                    <th scope="row"><label for="gath_motorcycles">Proizvodi povezani s motociklima</label></th>
                    <td>
                        <input type="checkbox" id="gath_motorcycles" name="gath_motorcycles" value="1"<?= $motorcycles ? ' checked' : ''; ?>>
                    </td>
                </tr>

                <tr>
                    <th scope="row"><label for="gath_login_for_prices">Korisnik mora biti ulogiran da bi vidio cijene i stanje zalihe</label></th>
                    <td>
                        <input type="checkbox" id="gath_login_for_prices" name="gath_login_for_prices" value="1"<?= get_option('gath_login_for_prices') ? ' checked' : ''; ?>>
                    </td>
                </tr>

                <tr>
                    <th scope="row"><label for="gath_backorders">Proizvode je moguće naručiti iako ih nema na stanju</label></th>
                    <td>
                        <input type="checkbox" id="gath_backorders" name="gath_backorders" value="1"<?= get_option('gath_backorders') ? ' checked' : ''; ?>>
                    </td>
                </tr>

                <tr>
                    <th scope="row"><label>Moduli</label></th>
                    <td>
                        <?php foreach (INTI_GATH_MODULES as $key => $label): ?>
                            <div>
                                <label>
                                    <input type="checkbox" name="gath_module_<?= $key; ?>" value="1"<?= get_option("gath_module_{$key}") ? ' checked' : ''; ?>>
                                    <?= $label; ?>
                                </label>
                            </div>

                            <br>
                        <?php endforeach; ?>
                    </td>
                </tr>
            </table>

            <?php
            settings_fields('gath_options');
            submit_button('Spremi postavke');
            ?>
        </form>
    </div>
    <?php
}
