
class MotorcycleSelectWidget {

    constructor(root) {
        this.root = root;
        this.currentManufacturer = this.manufacturers.value;
        this.currentModel = null;
        this.currentMotorcycle = null;

        this.manufacturers.addEventListener('change', this.onManufacturerChange);
        this.models.addEventListener('change', this.onModelChange);
        this.years.addEventListener('change', this.onMotorcycleChange);
    }

    get manufacturers() {
        if (this._manufacturers === undefined) {
            this._manufacturers = this.root.querySelector('select[name="manufacturer"]')
        }
        return this._manufacturers;
    }

    get models() {
        if (this._models === undefined) {
            this._models = this.root.querySelector('select[name="model"]');
        }
        return this._models;
    }

    get years() {
        if (this._years === undefined) {
            this._years = this.root.querySelector('select[name="motorcycle_id"]');
        }
        return this._years;
    }

    get submit() {
        if (this._submit === undefined) {
            this._submit = this.root.querySelector('button[type="submit"]');
        }
        return this._submit;
    }

    onManufacturerChange = (event) => {
        if (this.currentManufacturer !== event.target.value) {
            this.currentManufacturer = event.target.value;
            this.updateModels();
        }
    };

    onModelChange = (event) => {
        if (this.currentModel !== event.target.value) {
            this.currentModel = event.target.value;
            this.updateYears();
        }
    };

    onMotorcycleChange = (event) => {
        if (this.currentMotorcycle !== event.target.value) {
            this.currentMotorcycle = event.target.value;
            this.updateSubmit();
        }
    };

    updateModels = () => {
        this.currentModel = null;
        this.models.disabled = true;
        this.models.querySelectorAll('option').forEach((option, index) => {
            if (index > 0) {
                option.remove();
            }
        });

        this.updateYears();

        if (this.currentManufacturer) {
            const data = new FormData();
            data.set('action', 'get_moto_models');
            data.set('manufacturer', this.currentManufacturer);

            axios
                .post(IntiGathMotorcycleSelect.ajax_url, data)
                .then((response) => {
                    response.data.forEach((model) => {
                        const option = document.createElement('option');
                        option.setAttribute('value', model.id);
                        option.textContent = model.name;
                        this.models.appendChild(option);
                    });
                    this.models.disabled = false;
                });
        }
    };

    updateYears = () => {
        this.currentMotorcycle = null;
        this.years.disabled = true;
        this.years.querySelectorAll('option').forEach((option, index) => {
            if (index > 0) {
                option.remove();
            }
        });

        this.updateSubmit();

        if (this.currentModel) {
            const data = new FormData();
            data.set('action', 'get_moto_years');
            data.set('model', this.currentModel);

            axios
                .post(IntiGathMotorcycleSelect.ajax_url, data)
                .then((response) => {
                    response.data.forEach((year) => {
                        const option = document.createElement('option');
                        option.setAttribute('value', year.id);
                        option.textContent = year.year;
                        this.years.appendChild(option);
                    });
                    this.years.disabled = false;
                });
        }
    };

    updateSubmit = () => {
        this.submit.disabled = !this.currentMotorcycle;
    };
}

document.addEventListener('DOMContentLoaded', () => {
    const widgets = document.querySelectorAll('.widget_gath_motorcycle_select');

    widgets.forEach((widget) => {
        new MotorcycleSelectWidget(widget);
    });
});

