<?php
namespace Inti\Gath\Hydrator\Tire;

use Laminas\Hydrator\NamingStrategy\MapNamingStrategy;
use Laminas\Hydrator\ReflectionHydrator;
use Laminas\Hydrator\Strategy\ClosureStrategy;

/**
 * Class FilterValuesHydrator
 *
 * @package Inti\Gath\Hydrator\Tire
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class FilterValuesHydrator extends ReflectionHydrator
{
    /**
     * Konstruktor.
     */
    public function __construct()
    {
        $this->setNamingStrategy(MapNamingStrategy::createFromHydrationMap([
            'ArtApply' => 'vehicleTypes',
            'Season' => 'seasons',
            'SpeedIdx' => 'speedIndices',
            'BrandName' => 'brands',
            'LoadIndex1' => 'loadIndices',
            'TireWidth' => 'widths',
            'TireHeight' => 'heights',
            'TireDiameter' => 'diameters',
        ]));

        $this->addStrategy('vehicleTypes', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('seasons', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('speedIndices', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('brands', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('loadIndices', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('widths', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('heights', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
        $this->addStrategy('diameters', new ClosureStrategy(null, [$this, 'arrayFilterCallback']));
    }

    /**
     * @param $value
     * @return array
     */
    public function arrayFilterCallback($value)
    {
        return is_array($value) ? array_filter($value) : $value;
    }
}
