<?php
namespace Inti\Gath\Hydrator\Tire;

use Laminas\Hydrator\NamingStrategy\MapNamingStrategy;
use Laminas\Hydrator\ReflectionHydrator;
use Laminas\Hydrator\Strategy\ClosureStrategy;
use Laminas\Hydrator\Strategy\DateTimeFormatterStrategy;

/**
 * Class TireHydrator
 *
 * @package Inti\Gath\Hydrator\Tire
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class TireHydrator extends ReflectionHydrator
{
    /**
     * Konstruktor.
     */
    public function __construct()
    {
        $this->setNamingStrategy(MapNamingStrategy::createFromHydrationMap([
            'supplier_sku' => 'sku',
            'date_modified' => 'dateModified',
            'brand_name' => 'brand',
            'vehicle_type' => 'vehicleType',
            'tire_width' => 'width',
            'tire_height' => 'height',
            'tire_diameter' => 'diameter',
            'speed_idx' => 'speedIndex',
            'fuel_efficency' => 'fuelEfficiency',
            'wet_braking' => 'wetBraking',
            'load_index1' => 'loadIndex',
            'image' => 'images',
        ]));

        $this->addStrategy('dateModified', new DateTimeFormatterStrategy('Y-m-d H:i:s.u'));
        $this->addStrategy('images', new ClosureStrategy(null, function ($value) {
            return [$this->getExternalImageUrl($value)];
        }));
    }

    protected function getExternalImageUrl($image)
    {
        $urlParts = parse_url($image);

        // Bail if url not valid
        if ( !filter_var($urlParts['scheme'] . '://' . $urlParts['host'], FILTER_VALIDATE_URL) )
            return '';

        // If scheme is https, we can handle it by default
        if ( $urlParts['scheme'] === 'https')
            return $image;

        // Return string in this format:
        // external-host.tld/[path/]image.jpg
        return 'external-' . $urlParts['host'] . $urlParts['path'];
    }
}
