<?php
namespace Inti\Gath\Model;

use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\Hydrator\ReflectionHydrator;

/**
 * Class ConfigProvider
 *
 * @package Inti\Gath\Model
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ConfigProvider
{
    /**
     * @return array
     */
    public function __invoke(): array
    {
        return (new ConfigAggregator([
            \Laminas\Hydrator\ConfigProvider::class,

            new ArrayProvider([
                'dependencies' => $this->getDependencies(),
                'hydrators' => $this->getHydratorConfig(),
            ]),
        ]))->getMergedConfig();
    }

    /**
     * @return array
     */
    public function getDependencies(): array
    {
        return [
            'factories' => [
                Repository\RepositoryManager::class => Repository\RepositoryManagerFactory::class,
            ],
        ];
    }

    /**
     * @return array
     */
    public function getHydratorConfig(): array
    {
        return [
            'aliases' => [
                Entity\Attribute::class => Hydrator\AttributeHydrator::class,
                Entity\Category::class => Hydrator\CategoryHydrator::class,
                Entity\Product::class => Hydrator\ProductHydrator::class,
                Entity\ProductPrice::class => Hydrator\ProductPriceHydrator::class,
                Entity\ProductStock::class => Hydrator\ProductStockHydrator::class,
                Entity\Property::class => ReflectionHydrator::class,
            ],
            'factories' => [
                Hydrator\ProductHydrator::class => Hydrator\ProductHydratorFactory::class,
            ],
            'invokables' => [
                Hydrator\AttributeHydrator::class,
                Hydrator\CategoryHydrator::class,
                Hydrator\ProductPriceHydrator::class,
                Hydrator\ProductStockHydrator::class,
            ],
        ];
    }
}
