<?php
namespace Inti\Gath\Model\Entity;

use Inti\Gath\Api\Record\ProductPriceRecord;

/**
 * Cijena proizvoda.
 *
 * @package Inti\Gath\Model\Entity
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ProductPrice
{
    /** @var int */
    private $productId;
    /** @var string */
    private $productSku;
    /** @var string */
    private $currency;
    /** @var float */
    private $regularPrice = 0;
    /** @var float */
    private $salePrice = 0;
    /** @var bool */
    private $priceHidden = false;
    /** @var ProductPriceRecord */
    private $record;

    /**
     * @return int
     */
    public function getProductId(): int
    {
        return $this->productId;
    }

    /**
     * @return string
     */
    public function getProductSku(): string
    {
        return $this->productSku;
    }

    /**
     * @return string
     */
    public function getCurrency(): string
    {
        return $this->currency;
    }

    /**
     * @return bool
     */
    public function isOnSale(): bool
    {
        return $this->salePrice > 0;
    }

    /**
     * @return float
     */
    public function getPrice(): float
    {
        return $this->isOnSale() ? $this->getSalePrice() : $this->getRegularPrice();
    }

    /**
     * @return float
     */
    public function getRegularPrice(): float
    {
        return $this->regularPrice;
    }

    /**
     * @return float
     */
    public function getSalePrice(): float
    {
        return $this->salePrice;
    }

    /**
     * @return bool
     */
    public function isPriceHidden(): bool
    {
        return $this->priceHidden;
    }

    /**
     * @return ProductPriceRecord
     */
    public function getRecord(): ProductPriceRecord
    {
        return $this->record;
    }

    /**
     * @param ProductPriceRecord $record
     */
    public function setRecord(ProductPriceRecord $record): void
    {
        $this->record = $record;
    }
}
