<?php
namespace Inti\Gath\Model\Hydrator\NamingStrategy;

use Laminas\Hydrator\NamingStrategy\MapNamingStrategy;
use Laminas\Hydrator\NamingStrategy\NamingStrategyInterface;

/**
 * Class ProductStockNamingStrategy
 *
 * @package Inti\Gath\Model\Hydrator\NamingStrategy
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ProductStockNamingStrategy implements NamingStrategyInterface
{
    /** @var NamingStrategyInterface */
    private $innerStrategy;

    /**
     * Konstruktor.
     */
    public function __construct()
    {
        $this->innerStrategy = MapNamingStrategy::createFromHydrationMap([
            'ProductId' => 'productId',
            'ProductCode' => 'productSku',
            'PurchaseStep' => 'purchaseStep',
            'Stock' => 'stock',
        ]);
    }

    /**
     * @inheritDoc
     */
    public function hydrate(string $name, ?array $data = null): string
    {
        if (preg_match('/^Stock_\d+$/', $name)) {
            return 'stockByWarehouse';
        }

        return $this->innerStrategy->hydrate($name, $data);
    }

    /**
     * @inheritDoc
     */
    public function extract(string $name, ?object $object = null): string
    {
        return $this->innerStrategy->extract($name, $object);
    }
}
