<?php
namespace Inti\Gath\Model\Hydrator;

use Laminas\Hydrator\HydratorInterface;
use Laminas\Hydrator\NamingStrategy\MapNamingStrategy;
use Laminas\Hydrator\ReflectionHydrator;
use Laminas\Hydrator\Strategy\DateTimeFormatterStrategy;

/**
 * Class ProductHydrator
 *
 * @package Inti\Gath\Model\Hydrator
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ProductHydrator extends ReflectionHydrator
{
    /**
     * Konstruktor.
     *
     * @param HydratorInterface $attributeHydrator
     */
    public function __construct(HydratorInterface $attributeHydrator)
    {
        $this->setNamingStrategy(MapNamingStrategy::createFromHydrationMap([
            'Attributes' => 'attributes',
            'date_modified' => 'dateModified',
            'Images' => 'images',
            'categories' => 'categoryIds',
        ]));

        $this->addStrategy('id', new Strategy\ProductIdStrategy());
        $this->addStrategy('attributes', new Strategy\ProductAttributesStrategy($attributeHydrator));
        $this->addStrategy('dateModified', new DateTimeFormatterStrategy('Y-m-d H:i:s.u'));
        $this->addStrategy('images', new Strategy\ProductImagesStrategy());
    }
}
