<?php
namespace Inti\Gath\Model\Hydrator\Strategy;

use Laminas\Hydrator\Strategy\StrategyInterface;

/**
 * Class ProductPriceStrategy
 *
 * @package Inti\Gath\Model\Hydrator\Strategy
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
abstract class ProductPriceStrategy implements StrategyInterface
{
    /**
     * @inheritDoc
     */
    public function extract($value, ?object $object = null)
    {
        return $value;
    }

    /**
     * @param array $data
     * @return bool
     */
    protected function hasDiscount(array $data): bool
    {
        return count($data['Discount'] ?? []) > 0;
    }

    /**
     * @param array $data
     * @return array
     */
    protected function getBestDiscount(array $data): array
    {
        foreach ($data['Discount'] as $discount) {
            if (!isset($bestDiscount) || $discount['Rate'] > $bestDiscount['Rate']) {
                $bestDiscount = $discount;
            }
        }

        return $bestDiscount ?? [];
    }

    /**
     * @param array $data
     * @return bool
     */
    protected function hasRebate(array $data): bool
    {
        return count($data['RabPolicy'] ?? []) > 0;
    }

    /**
     * @param array $data
     * @return array
     */
    protected function getBestRebate(array $data): array
    {
        foreach ($data['RabPolicy'] as $rebate) {
            $rebate = $rebate->toArray();
            if (!isset($bestRebate) || $rebate['Rate'] > $bestRebate['Rate']) {
                $bestRebate = $rebate;
            }
        }

        return $bestRebate ?? [];
    }
}
