<?php
namespace Inti\Gath\Model\Repository;

use Inti\Gath\Api\Endpoint\GetProductPrice;
use Inti\Gath\Api\Query\GetProductPriceQuery;
use Inti\Gath\Model\Entity\ProductPrice;
use function wp_get_current_user;

/**
 * Class ProductPriceRepository
 *
 * @package Inti\Gath\Model\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ProductPriceRepository extends AbstractRepository
{
    /**
     * @inheritDoc
     */
    public function getAll(): array
    {
        return [];
    }

    /**
     * @param array $productSkus
     * @param array $tireIds
     * @return ProductPrice[]
     */
    public function getByProductSku(array $productSkus = [], array $tireIds = []): array
    {
        $query = new GetProductPriceQuery();
        $query->PriceRabatePolicy = defined('GATH_WHOLESALE') && GATH_WHOLESALE === true;

        $user = wp_get_current_user();

        if ($user->ID > 0) {
            $query->Username = $user->nickname;
        }

        if (!empty($productSkus)) {
            $query->Codes = $productSkus;
        }

        if (!empty($tireIds)) {
            $query->TireIds = $tireIds;
        }

        /** @var GetProductPrice $endpoint */
        $endpoint = $this->endpoints->get(GetProductPrice::class);
        $result = $endpoint->call($query);

        foreach ($result->Results as $priceRecord) {
            $data = $priceRecord->toArray();
            /** @var ProductPrice $productPrice */
            $productPrice = $this->hydrator->hydrate($data, new ProductPrice());
            $productPrice->setRecord($priceRecord);
            $productPrices[] = $productPrice;
        }

        return $productPrices ?? [];
    }
}
