<?php
namespace Inti\Gath\Model\Repository;

use Inti\Gath\Api\Endpoint\Service\EndpointManager;
use Laminas\Hydrator\DelegatingHydrator;
use Laminas\Hydrator\HydratorInterface;
use Laminas\Hydrator\HydratorPluginManager;
use Laminas\ServiceManager\PluginManagerInterface;
use Psr\Container\ContainerInterface;

/**
 * Class ProductRespositoryFactory
 *
 * @package Inti\Gath\Model\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ProductRepositoryFactory
{
    /**
     * @param ContainerInterface $container
     * @return RepositoryInterface
     */
    public function __invoke(ContainerInterface $container): RepositoryInterface
    {
        /** @var EndpointManager $endpoints */
        $endpoints = $container->get(EndpointManager::class);

        /** @var PluginManagerInterface $hydrators */
        $hydrators = $container->get(HydratorPluginManager::class);
        /** @var HydratorInterface $hydrator */
        $hydrator = $hydrators->get(DelegatingHydrator::class);

        /** @var RepositoryManager $repos */
        $repos = $container->get(RepositoryManager::class);
        /** @var AttributeRepository $attributes */
        $attributes = $repos->get(AttributeRepository::class);
        /** @var CategoryRepository $categories */
        $categories = $repos->get(CategoryRepository::class);

        return new ProductRepository($endpoints, $hydrator, $attributes, $categories);
    }
}
