<?php
namespace Inti\Gath\Model\Repository;

use Inti\Gath\Api\Endpoint\GetProductStock;
use Inti\Gath\Api\Query\GetProductStockQuery;
use Inti\Gath\Model\Entity\ProductStock;

/**
 * Class StockInfoRepository
 *
 * @package Inti\Gath\Model\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ProductStockRepository extends AbstractRepository
{
    /** @var ProductStock[] */
    private $bySku = [];

    /**
     * @inheritDoc
     */
    public function getAll(): array
    {
        return [];
    }

    /**
     * @param array $productSkus
     * @param array $tireIds
     * @param bool $byWarehouse
     * @return ProductStock[]
     */
    public function getByProductSku(array $productSkus = [], array $tireIds = [], bool $byWarehouse = false): array
    {
        $query = new GetProductStockQuery();
        $query->StockByWarehouses = $byWarehouse;

        if (!empty($productSkus)) {
            $query->Codes = $productSkus;
        }

        if (!empty($tireIds)) {
            $query->TireIds = $tireIds;
        }

        /** @var GetProductStock $endpoint */
        $endpoint = $this->endpoints->get(GetProductStock::class);
        $result = $endpoint->call($query);

        foreach ($result->Results as $stockRecord) {
            $data = $stockRecord->toArray();
            $productStocks[] = $this->hydrator->hydrate($data, new ProductStock());
        }

        return $productStocks ?? [];
    }
}
