<?php
namespace Inti\Gath\Model\Repository;

use Interop\Container\ContainerInterface;
use Inti\Gath\Api\Endpoint\Service\EndpointManager;
use Laminas\Hydrator\DelegatingHydrator;
use Laminas\Hydrator\HydratorInterface;
use Laminas\Hydrator\HydratorPluginManager;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Laminas\ServiceManager\PluginManagerInterface;

/**
 * Class RepositoryFactory
 *
 * @package Inti\Gath\Model\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class RepositoryFactory implements FactoryInterface
{
    /**
     * @inheritDoc
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null): RepositoryInterface
    {
        /** @var EndpointManager $endpoints */
        $endpoints = $container->get(EndpointManager::class);
        /** @var PluginManagerInterface $hydrators */
        $hydrators = $container->get(HydratorPluginManager::class);
        /** @var HydratorInterface $hydrator */
        $hydrator = $hydrators->get(DelegatingHydrator::class);

        return new $requestedName($endpoints, $hydrator);
    }
}
