<?php
namespace Inti\Gath\Plugin;

/**
 * Class MotorcyclePlugin
 *
 * @package Inti\Gath\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class MotorcyclePlugin
{
    /**
     * Konstruktor.
     */
    public function __construct()
    {
        add_filter('query_vars', [$this, 'query_vars']);
        add_filter('rewrite_rules_array', [$this, 'rewrite_rules']);

        add_action('template_redirect', [$this, 'handle_selection']);
        add_action('template_redirect', [$this, 'maybe_redirect']);
    }

    public function query_vars(array $query_vars): array
    {
        $query_vars[] = 'motorcycle_id';
        return $query_vars;
    }

    public function rewrite_rules(array $rules): array
    {
        $shopPageId = wc_get_page_id('shop');
        $shopPage = get_post($shopPageId);

        $shopPageSlug = $shopPage->post_name;
        $categoryPageSlug = get_option('woocommerce_product_category_slug') ?: _x('product-category', 'slug', 'woocommerce');

        foreach ($rules as $regex => $rule) {
            if (preg_match('/^(' . $shopPageSlug . '|' . $categoryPageSlug . ')/', $regex)) {
                $regex = '(\d+)/' . $regex;
                $rule = preg_replace_callback('/\$matches\[(\d+)]/', function(array $matches) {
                    return sprintf('$matches[%d]', intval($matches[1]) + 1);
                }, $rule);

                $newRules[$regex] = $rule . '&motorcycle_id=$matches[1]';
            }
        }

        $rules = array_merge($newRules ?? [], $rules);

        return $rules;
    }

    public function handle_selection()
    {
        global $wp_query;

        if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['inti_motorcycle_select'])) {
            setcookie('inti_motorcycle_id', $_POST['motorcycle_id'], time() + 365 * 86400, '/');
            $_COOKIE['inti_motorcycle_id'] = $_POST['motorcycle_id'];
            $wp_query->query_vars['motorcycle_id'] = null;
        } else if (!empty($wp_query->query_vars['motorcycle_id']) && $wp_query->query_vars['motorcycle_id'] !== ($_COOKIE['inti_motorcycle_id'] ?? '')) {
            setcookie('inti_motorcycle_id', $wp_query->query_vars['motorcycle_id'], time() + 365 * 86400, '/');
            $_COOKIE['inti_motorcycle_id'] = $wp_query->query_vars['motorcycle_id'];
        }
    }

    public function maybe_redirect()
    {
        global $wp_query;

        if (!isset($_COOKIE['inti_motorcycle_id'])) {
            return;
        }

        if (!is_shop() && !is_product_category()) {
            return;
        }

        if (($wp_query->query_vars['motorcycle_id'] ?? '') === $_COOKIE['inti_motorcycle_id']) {
            return;
        }

        if (is_shop()) {
            $shopPageId = wc_get_page_id('shop');
            $permalink = get_permalink($shopPageId);
        } else if (is_product_category()) {
            $permalink = get_term_link(get_queried_object_id());
        }

        $permalink = preg_replace('#^' . WP_HOME . '(/\d+)?#', WP_HOME . '/' . $_COOKIE['inti_motorcycle_id'], $permalink);
        wp_redirect($permalink);
        exit;
    }
}
