<?php
namespace Inti\Gath\Repository;

use Inti\Gath\Entity\Moto\Manufacturer;
use Inti\Gath\Entity\Moto\Model;
use Inti\Gath\Entity\Moto\Motorcycle;

/**
 * Class MotoRepository
 *
 * @package Inti\Gath\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class MotoRepository extends AbstractRepository
{
    /**
     * @return Manufacturer[]
     */
    public function getManufacturers(): array
    {
        $manufacturers = get_transient('gath_moto_manufacturers');

        if (empty($manufacturers)) {
            $results = $this->client->request('GET', 'moto/brands');

            foreach ($results['Results'] ?? [] as $result) {
                /** @var Manufacturer $product */
                $manufacturers[] = $this->hydrator->hydrate($result, new Manufacturer());
            }

            set_transient('gath_moto_manufacturers', $manufacturers, HOUR_IN_SECONDS);
        }

        return $manufacturers ?: [];
    }

    /**
     * @param int $manufacturerId
     * @return Model[]
     */
    public function getModels(int $manufacturerId): array
    {
        $models = get_transient('gath_moto_models_' . $manufacturerId);

        if (empty($models)) {
            $results = $this->client->request('POST', 'moto/models', [
                'Brands' => [$manufacturerId],
            ]);

            foreach ($results['Results'] ?? [] as $result) {
                /** @var Model $product */
                $models[] = $this->hydrator->hydrate($result, new Model());
            }

            set_transient('gath_moto_models_' . $manufacturerId, $models, HOUR_IN_SECONDS);
        }

        return $models ?: [];
    }

    /**
     * @param int $modelId
     * @return Motorcycle[]
     */
    public function getMotorcycles(int $modelId): array
    {
        $motorcycles = get_transient('gath_moto_motorcycles_' . $modelId);

        if (empty($motorcycles)) {
            $results = $this->client->request('POST', 'moto/motorcycles', [
                'Models' => [$modelId],
            ]);

            foreach ($results['Results'] ?? [] as $result) {
                /** @var Model $product */
                $motorcycles[] = $this->hydrator->hydrate($result, new Motorcycle());
            }

            set_transient('gath_moto_motorcycles_' . $modelId, $motorcycles, HOUR_IN_SECONDS);
        }

        return $motorcycles;
    }

    /**
     * @param int $id
     * @return Motorcycle
     */
    public function getMotorcycle(int $id): Motorcycle
    {
        $motorcycle = get_transient('gath_motorcycle_' . $id);

        if (empty($motorcycle)) {
            $results = $this->client->request('GET', 'moto/motorcycles', [
                'Id' => $id,
            ]);

            $motorcycle = $this->hydrator->hydrate($results['Results'][0], new Motorcycle());
            set_transient('gath_motorcycle_' . $id, $motorcycle, HOUR_IN_SECONDS);
        }

        return $motorcycle;
    }
}
