<?php
namespace Inti\Gath\Widget;

use Inti\Gath\Entity\Moto\Model;
use Inti\Gath\Entity\Moto\Motorcycle;
use Inti\Gath\Repository\MotoRepository;
use WC_Widget;

/**
 * Class MotorcycleSelect
 *
 * @package Inti\Gath\Widget
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class MotorcycleSelect extends WC_Widget
{
    /** @var MotoRepository */
    private $repo;

    /**
     * Konstruktor.
     *
     * @param MotoRepository $repo
     */
    public function __construct(MotoRepository $repo)
    {
        $this->repo = $repo;

        $this->widget_cssclass    = 'woocommerce widget_gath_motorcycle_select';
        $this->widget_description = 'Obrazac za odabir motocikla';
        $this->widget_id          = 'inti_gath_motorcycle_select';
        $this->widget_name        = 'Odabir motocikla';
        $this->settings           = array(
            'title'              => array(
                'type'  => 'text',
                'std'   => 'Odabir motocikla',
                'label' => 'Naslov',
            ),
        );

        parent::__construct();

        add_action('wp_enqueue_scripts', [$this, 'wp_enqueue_scripts']);

        add_action('wp_ajax_get_moto_models', [$this, 'get_models']);
        add_action('wp_ajax_nopriv_get_moto_models', [$this, 'get_models']);
        add_action('wp_ajax_get_moto_years', [$this, 'get_years']);
        add_action('wp_ajax_nopriv_get_moto_years', [$this, 'get_years']);
    }

    public function wp_enqueue_scripts()
    {
        wp_register_script('inti-gath-motorcycle-select', INTI_GATH_URL . '/js/motorcycle-select.js', ['axios'], false, true);
        wp_localize_script('inti-gath-motorcycle-select', 'IntiGathMotorcycleSelect', [
            'ajax_url' => admin_url('admin-ajax.php'),
        ]);
    }

    public function get_models()
    {
        $models = $this->repo->getModels(intval($_POST['manufacturer']));
        $output = array_map(function (Model $model) {
            return [
                'id' => $model->getId(),
                'name' => $model->getName(),
            ];
        }, $models);

        echo json_encode($output);
        wp_die();
    }

    public function get_years()
    {
        $motorcycles = $this->repo->getMotorcycles(intval($_POST['model']));
        $years = array_map(function (Motorcycle $motorcycle) {
            return [
                'id' => $motorcycle->getId(),
                'year' => $motorcycle->getYear()
            ];
        }, $motorcycles);

        echo json_encode($years);
        wp_die();
    }

    public function widget($args, $instance)
    {
        global $wp_query;

        $manufacturers = $this->repo->getManufacturers();
        wp_enqueue_script('inti-gath-motorcycle-select');

        if (isset($wp_query->query_vars['motorcycle_id']) && !empty($wp_query->query_vars['motorcycle_id'])) {
            $motorcycle = $this->repo->getMotorcycle(intval($wp_query->query_vars['motorcycle_id']));

            $motorcycleId = $motorcycle->getId();
            $manufacturerId = $motorcycle->getManufacturerId();
            $modelId = $motorcycle->getModelId();

            $models = $this->repo->getModels($manufacturerId);
            $motorcycles = $this->repo->getMotorcycles($modelId);
        }

        $this->widget_start($args, $instance);
        ?>
        <form method="post">
            <p>
                <select name="manufacturer">
                    <option value="">Odaberite proizvođača...</option>

                    <?php foreach ($manufacturers as $manufacturer): ?>
                        <option value="<?= $manufacturer->getId(); ?>"<?=
                            isset($manufacturerId) && $manufacturerId === $manufacturer->getId() ? ' selected' : '';
                        ?>>
                            <?= $manufacturer->getName(); ?>
                        </option>
                    <?php endforeach; ?>
                </select>
            </p>

            <p>
                <select name="model"<?= !isset($models) ? ' disabled' : ''; ?>>
                    <option value="">Odaberite model...</option>

                    <?php foreach ($models ?? [] as $model): ?>
                        <option value="<?= $model->getId(); ?>"<?=
                            isset($modelId) && $modelId === $model->getId() ? ' selected' : '';
                        ?>>
                            <?= $model->getName(); ?>
                        </option>
                    <?php endforeach; ?>
                </select>
            </p>

            <p>
                <select name="motorcycle_id"<?= !isset($motorcycles) ? ' disabled' : ''; ?>>
                    <option value="">Odaberite godinu...</option>

                    <?php foreach ($motorcycles ?? [] as $motorcycle): ?>
                        <option value="<?= $motorcycle->getId(); ?>"<?=
                            isset($motorcycleId) && $motorcycleId === $motorcycle->getId() ? ' selected' : '';
                        ?>>
                            <?= $motorcycle->getYear(); ?>
                        </option>
                    <?php endforeach; ?>
                </select>
            </p>

            <button type="submit" class="button" name="inti_motorcycle_select"<?= !isset($motorcycleId) ? ' disabled' : ''; ?>>
                Odaberi!
            </button>
        </form>
        <?php
        $this->widget_end($args);
    }
}
