<?php
namespace Inti\Gath\Wp;

/**
 * Class ImageManager
 *
 * @package Inti\Gath\Wp
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class ImageManager
{
    /** @var string */
    private $basePath;

    /**
     * Konstruktor.
     *
     * @param string $basePath
     */
    public function __construct(string $basePath)
    {
        $this->basePath = rtrim($basePath, '/');

        if (!is_dir($this->basePath)) {
            mkdir($basePath, 0777, true);
        }
    }

    /**
     * @param string $gathPath
     * @param string $image
     */
    public function uploadImage(string $gathPath, string $image)
    {
        $imageHash = md5($image);
        $localPath = 'gath/' . preg_replace('/(\.[^.]+)$/', '.' . $imageHash . '\\1', ltrim($gathPath, '/'));
        $fullPath = $this->basePath . '/' . $localPath;

        $dirname = dirname($fullPath);
        if (!is_dir($dirname)) {
            mkdir($dirname, 0777, true);
        }

        $attachmentId = $this->getAttachmentIdByGathPath($gathPath);

        if ($attachmentId === 0) {

            file_put_contents($fullPath, $image);

            $attachmentId = wp_insert_attachment([
                'file' => $fullPath,
                'post_title' => basename($gathPath),
                'post_content' => '',
                'post_status' => 'publish',
                'post_mime_type' => 'image/jpeg',
            ]);

            update_post_meta($attachmentId, '_gath_path', $gathPath);
        } else {
            if (get_post_meta($attachmentId, '_image_hash') === $imageHash) {
                return;
            }

            $oldLocalPath = ltrim(get_post_meta($attachmentId, '_wp_attached_file', true), '/');
            $this->deleteImage($oldLocalPath);

            file_put_contents($fullPath, $image);
            update_post_meta($attachmentId, '_wp_attached_file', $localPath);
        }

        require_once ABSPATH . 'wp-admin/includes/image.php';
        $metaData = wp_generate_attachment_metadata($attachmentId, $fullPath);
        wp_update_attachment_metadata($attachmentId, $metaData);

        update_post_meta($attachmentId, '_image_hash', $imageHash);
    }

    /**
     * Vraća ID posta za sliku s datom Gath putanjom.
     *
     * @param string $gathPath
     * @return int
     */
    public function getAttachmentIdByGathPath(string $gathPath): int
    {
        $gathPath = preg_replace('#/#', '__SLASH__', $gathPath);
        $gathPath = rawurlencode($gathPath);
        $gathPath = preg_replace('/__SLASH__/', '/', $gathPath);

        $attachments = get_posts([
            'post_type' => 'attachment',
            'meta_query' => [[
                'key' => '_gath_path',
                'value' => $gathPath,
                'compare' => '=',
            ]],
        ]);

        if (count($attachments) > 0) {
            return $attachments[0]->ID;
        }

        return 0;
    }

    public function getThumbnail(string $localPath, int $width, int $height, bool $crop): string
    {
        /*$localPath = explode('.', $localPath);
        $localPath[count($localPath) - 1] = strtolower($localPath[count($localPath) - 1]);
        $localPath = implode('.', $localPath);*/
        $localPath = explode('/', $localPath);
        $localPath[count($localPath) - 1] = rawurlencode($localPath[count($localPath) - 1]);
        $localPath = implode('/', $localPath);
        $thumbLocalPath = 'thumbnails/' . ($crop ? 'cropped/' : '') . intval($width) . 'x' . intval($height) . '/' . $localPath;
        $thumbFullPath = $this->basePath . '/' . $thumbLocalPath;

        if (!file_exists($thumbFullPath)) {
            $thumbDirname = dirname($thumbFullPath);

            if (!is_dir($thumbDirname)) {
                mkdir($thumbDirname, 0777, true);
            }

            $fullPath = $this->basePath . '/' . $localPath;
            /** @var ImageEditor $editor */
            $editor = wp_get_image_editor($fullPath);

            if (is_wp_error($editor)) {
                return $localPath;
            }

            $thumb = $editor->make_subsize([
                'width' => $width,
                'height' => $height,
                'crop' => $crop,
            ], $thumbFullPath);

            if (is_wp_error($thumb)) {
                return $localPath;
            }
        }

        return $thumbLocalPath;
    }

    /**
     * Briše sliku i njene minijature.
     *
     * @param string $localPath
     */
    public function deleteImage(string $localPath)
    {
        unlink($this->basePath . '/' . $localPath);
        $this->deleteThumbnails($localPath);
    }

    public function deleteThumbnails(string $localPath)
    {
        foreach (['thumbnails/', 'thumbnails/cropped/'] as $dir) {
            $currentDir = $this->basePath . '/' . $dir;

            foreach (scandir($currentDir) as $size) {
                if (!preg_match('/^\d/', $size)) {
                    continue;
                }

                $filename = $currentDir . '/' . $size . '/' . $localPath;

                if (file_exists($filename)) {
                    unlink($filename);
                }
            }
        }
    }
}
