<?php
namespace Inti\Gath\Wp\Plugin;

use Elementor\Group_Control_Image_Size;
use Inti\Gath\Wp\ImageEditor;
use Inti\Gath\Wp\ImageManager;
use Inti\WordPress\Plugin\PluginInterface;

/**
 * Class GathImages
 *
 * @package Inti\Gath\Wp\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class GathImages implements PluginInterface
{
    /** @var ImageManager */
    private $manager;

    /**
     * Konstruktor.
     *
     * @param ImageManager $manager
     */
    public function __construct(ImageManager $manager)
    {
        $this->manager = $manager;
    }

    /**
     * @inheritDoc
     */
    public function init()
    {
        add_filter('image_downsize', [$this, 'image_downsize'], 10, 3);
        add_filter('wp_image_editors', [$this, 'wp_image_editors']);
        add_action('delete_attachment', [$this, 'delete_attachment']);
        add_filter('rewrite_rules_array', [$this, 'rewrite_rules']);
        add_filter('query_vars', [$this, 'query_vars']);
        add_action('template_redirect', [$this, 'template_redirect']);
    }

    public function rewrite_rules(array $rules): array
    {
        $new_rules['^external/(.+)'] = 'index.php?external_image=$matches[1]';
        return array_merge($new_rules, $rules);
    }

    public function query_vars(array $query_vars): array
    {
        $query_vars[] = 'external_image';
        return $query_vars;
    }

    public function template_redirect()
    {
        global $wp_query;

        if (isset($wp_query->query_vars['external_image']) && !empty($wp_query->query_vars['external_image'])) {
            header('Content-Type: image/jpeg');
            echo file_get_contents('http://' . $wp_query->query_vars['external_image']);
            exit;
        }
    }

    public function wp_image_editors(array $editors): array
    {
        array_unshift($editors, ImageEditor::class);
        return $editors;
    }

    public function delete_attachment(int $attachmentId)
    {
        $localPath = get_post_meta($attachmentId, '_wp_attached_file', true);
        $this->manager->deleteThumbnails($localPath);
    }

    public function image_downsize($false, $attachment_id, $size)
    {
        if ($size === 'full') {
            return $false;
        }

        if (is_array($size)) {
            $width = $size[0] ?? 0;
            $height = $size[1] ?? 0;
            $crop = false;
        } else {
            $imageSizes = Group_Control_Image_Size::get_all_image_sizes();

            if (!isset($imageSizes[$size])) {
                return $false;
            }

            $width = $imageSizes[$size]['width'];
            $height = $imageSizes[$size]['height'];
            $crop = (bool) $imageSizes[$size]['crop'];
        }

        $imagePath = get_post_meta($attachment_id, '_wp_attached_file', true);


        $thumbPath = $this->manager->getThumbnail($imagePath, $width, $height, $crop);
        $lcThumbPath = preg_replace('/\.JPG$/', '.jpg', $thumbPath);
        $uploads = wp_upload_dir();

        if (!file_exists($uploads['path'] . '/' . $thumbPath) && file_exists($uploads['path'] . '/' . $lcThumbPath)) {
            $thumbPath = $lcThumbPath;
        }

        $thumbUrl = $uploads['url'] . '/' . $thumbPath;

        return [$thumbUrl, $width, $height, true];
    }
}
