<?php
namespace Inti\Gath\Wp\Plugin;

use Inti\Gath\Model\Hydrator\Strategy\IdStrategy;
use Inti\WordPress\Plugin\PluginInterface;

/**
 * Class GathMetadata
 *
 * @package Inti\Gath\Wp\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
abstract class GathMetadata implements PluginInterface
{
    /** @var string */
    protected $metaType = '';
    /** @var string[] */
    protected $metaKeys = [];
    /** @var array<string, mixed> */
    private $metadata = [];

    /**
     * @inheritDoc
     */
    public function init(): void
    {
        add_filter("get_{$this->metaType}_metadata", [$this, 'get_metadata'], 10, 3);
    }

    /**
     * @param array|null $metadata
     * @param int $objectId
     * @param string $metaKey
     * @return array|null
     */
    public function get_metadata($metadata, int $objectId, string $metaKey)
    {
        if (!empty($metaKey) && !in_array($metaKey, $this->metaKeys) || $objectId < IdStrategy::ID_OFFSET) {
            return $metadata;
        }

        if ($objectId < IdStrategy::ID_OFFSET && !is_array($metadata)) {
            $metadata = $this->get_local_metadata($objectId);
        }

        if (!isset($this->metadata[$objectId])) {
            $this->metadata[$objectId] = $this->get_metadata_by_id($objectId);

            foreach ($this->metadata[$objectId] as &$value) {
                if (!is_array($value)) {
                    $value = [$value];
                }
            }
        }

        return !empty($metaKey)
            ? $this->metadata[$objectId][$metaKey]
            : (
                is_array($metadata)
                    ? array_merge($metadata, $this->metadata[$objectId] ?? [])
                    : $this->metadata[$objectId]
            );
    }

    /**
     * @param int $objectId
     * @return array
     * @noinspection PhpUnusedParameterInspection
     */
    protected function get_metadata_by_id(int $objectId): array
    {
        return [];
    }

    /**
     * @param int $objectId
     * @return array
     */
    private function get_local_metadata(int $objectId): array
    {
        $metadata = wp_cache_get($objectId, "{$this->metaType}_meta");

        if (!$metadata) {
            $metadata = update_meta_cache($this->metaType, [$objectId]);
            $metadata = $metadata[$objectId] ?? null;
        }

        return $metadata;
    }
}
