<?php
namespace Inti\Gath\Wp\Plugin;

use Inti\Gath\Model\Repository\RepositoryManager;
use Inti\Gath\Model\Repository\ProductRepository;
use Inti\Gath\Wp\ImageManager;
use Inti\WordPress\Plugin\PluginInterface;
use Psr\Container\ContainerInterface;
use Solarium\Client;

/**
 * Class GathProductsFactory
 *
 * @package Inti\Gath\Wp\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class GathProductsFactory
{
    /**
     * @param ContainerInterface $container
     * @return PluginInterface
     */
    public function __invoke(ContainerInterface $container): PluginInterface
    {
        /** @var RepositoryManager $repos */
        $repos = $container->get(RepositoryManager::class);
        /** @var ProductRepository $repo */
        $repo = $repos->get(ProductRepository::class);
        /** @var ImageManager $imageManager */
        $imageManager = $container->get(ImageManager::class);

        if (defined('SOLR_HOST') && defined('SOLR_PORT') && defined('SOLR_PATH')) {
            $solr = new Client([
                'endpoint' => [
                    'webshop' => [
                        'host' => SOLR_HOST,
                        'port' => SOLR_PORT,
                        'path' => SOLR_PATH,
                    ],
                ],
            ]);
        }

        return new GathProducts($repo, $imageManager, $solr ?? null);
    }
}
