<?php
namespace Inti\Gath\Wp\Plugin;

use Inti\Gath\Model\Hydrator\Strategy\ProductIdStrategy;
use Inti\Gath\Model\Hydrator\Strategy\TireIdStrategy;
use Inti\Gath\Model\Repository\ProductStockRepository;
use Inti\Gath\Model\Repository\ProductRepository;

/**
 * Class GathStock
 *
 * @package Inti\Gath\Wp\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class GathStock extends GathMetadata
{
    /** @var ProductStockRepository */
    private $stocks;
    /** @var ProductRepository */
    private $products;
    /** @var string */
    protected $metaType = 'post';
    /** @var string[] */
    protected $metaKeys = [
        '_backorders',
        '_manage_stock',
        '_stock',
        '_stock_status',
    ];

    /**
     * Konstruktor.
     *
     * @param ProductStockRepository $stocks
     * @param ProductRepository $products
     */
    public function __construct(ProductStockRepository $stocks, ProductRepository $products)
    {
        $this->stocks = $stocks;
        $this->products = $products;

        remove_action( 'woocommerce_checkout_order_created', 'wc_reserve_stock_for_order' );

        remove_action( 'woocommerce_payment_complete', 'wc_maybe_reduce_stock_levels' );
        remove_action( 'woocommerce_order_status_completed', 'wc_maybe_reduce_stock_levels' );
        remove_action( 'woocommerce_order_status_processing', 'wc_maybe_reduce_stock_levels' );
        remove_action( 'woocommerce_order_status_on-hold', 'wc_maybe_reduce_stock_levels' );
    }

    /**
     * @param int $objectId
     * @return array
     */
    protected function get_metadata_by_id(int $objectId): array
    {
        if ($objectId > TireIdStrategy::ID_OFFSET) {
            $productStock = $this->stocks->getByProductSku([], [$objectId])[0] ?? null;
        } else if ($objectId > ProductIdStrategy::ID_OFFSET) {
            $product = $this->products->getById($objectId);
            $productStock = $this->stocks->getByProductSku([$product->getSku()])[0] ?? null;
        } else {
            $sku = get_post_meta($objectId, '_sku', true);
            $productStock = $this->stocks->getByProductSku([$sku])[0] ?? null;
        }

        // lokalni proizvodi
        if ($objectId < ProductIdStrategy::ID_OFFSET) {
            if ($sku = get_post_meta($objectId, '_sku', true)) {
                $productStock = $this->stocks->getByProductSku([$sku])[0] ?? null;
            }
        }

        $backorders = get_option('gath_backorders');

        return isset($productStock) ? [
            '_backorders' => $backorders ? 'yes' : 'no',
            '_manage_stock' => 'yes',
            '_stock' => $productStock->getStock(),
            '_stock_status' => $productStock->getStock() > 0 ? 'instock' : ($backorders ? 'onbackorder' : 'outofstock'),
        ] : [];
    }
}
