<?php
namespace Inti\Gath\Wp\Plugin;

use WP_Term;
use WP_Term_Query;

/**
 * Class AbstractTermProvider
 *
 * @package Inti\Gath\Wp\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
abstract class GathTerms extends GathMetadata
{
    /** @var string */
    protected $taxonomy;

    /**
     * @inheritDoc
     */
    public function init(): void
    {
        parent::init();

        add_action('admin_init', [$this, 'admin_init']);
        add_filter('terms_pre_query', [$this, 'get_terms'], 10, 2);
    }

    public function admin_init()
    {
        if (is_numeric($_GET['tag_ID'] ?? '')) {
            $this->prepare_terms($this->get_terms_by_id([intval($_GET['tag_ID'])]));
        }
    }

    /**
     * Vraća true ako ovaj provider pruža pojmove za datu taksonomiju.
     *
     * @param array $taxonomies
     * @return bool
     */
    protected function is_valid_taxonomy(array $taxonomies): bool
    {
        return in_array($this->taxonomy, $taxonomies);
    }

    /**
     * Vraća taksonomije čije pojmove tražimo.
     *
     * @param WP_Term_Query $query
     * @return array|mixed
     */
    protected function get_queried_taxonomies(WP_Term_Query $query)
    {
        return $query->query_vars['taxonomy'] ?? [];
    }

    /**
     * @param $terms
     * @param WP_Term_Query $query
     * @return WP_Term[]|mixed
     */
    public function get_terms($terms, WP_Term_Query $query)
    {
        if (!empty($terms) || !$this->is_valid_taxonomy($this->get_queried_taxonomies($query))) {
            return $terms;
        }

        switch (true) {
            case !empty($query->query_vars['include']):
                $categoryIds = $query->query_vars['include'];

                if (!is_array($categoryIds)) {
                    $categoryIds = array_map('intval',  explode(',', $query->query_vars['include']));
                }

                $terms = $this->get_terms_by_id(array_unique($categoryIds));
                break;
            case !empty($query->query_vars['slug']):
                $slugs = (array) $query->query_vars['slug'];
                $terms = $this->get_terms_by_slug($slugs);
                break;
            case is_int($query->query_vars['parent']):
                $terms = $this->get_terms_by_parent_id($query->query_vars['parent']);
                break;
            default:
                $terms = $this->get_all_terms($query);
        }


        return $this->prepare_terms($terms, $query);
    }

    /**
     * @param WP_Term_Query $query
     * @return WP_Term[]
     * @noinspection PhpUnusedParameterInspection
     */
    protected function get_all_terms(WP_Term_Query $query): array
    {
        return [];
    }

    /**
     * @param array $term_ids
     * @return WP_Term[]
     * @noinspection PhpUnusedParameterInspection
     */
    protected function get_terms_by_id(array $term_ids): array
    {
        return [];
    }

    /**
     * @param int $parent_id
     * @return WP_Term[]
     */
    protected function get_terms_by_parent_id(int $parent_id): array
    {
        return [];
    }

    /**
     * @param string[] $slugs
     * @return WP_Term[]
     */
    protected function get_terms_by_slug(array $slugs): array
    {
        return [];
    }

    /**
     * @param int $id
     * @return WP_Term|null
     */
    protected function get_term_by_id(int $id): ?WP_Term
    {
        $terms = $this->get_terms_by_id([$id]);
        return $terms[0] ?? null;
    }

    /**
     * @param WP_Term[] $terms
     * @param WP_Term_Query $query
     * @return mixed
     */
    private function prepare_terms(array $terms, WP_Term_Query $query = null)
    {
        foreach ($terms as $term) {
            wp_cache_set($term->term_id, $term, 'terms');
        }

        if ($query instanceof WP_Term_Query) {
            switch ($query->query_vars['fields']) {
                case 'count':
                    $return = count($terms);
                    break;
                case 'ids':
                    $return = array_map(function(WP_Term $term) {
                        return intval($term->term_id);
                    }, $terms);
                    break;
                case 'id=>parent':
                    foreach ($terms as $term) {
                        $return[$term->term_id] = $term->parent;
                    }
                    break;
            }
        }

        return $return ?? $terms;
    }
}
