<?php
namespace Inti\Gath\Wp\Widget;

use Inti\Gath\Model\Repository\TireRepository;
use WC_Widget;
use WP_Term;

/**
 * Class TireFilter
 *
 * @package Inti\Gath\Widget
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-gath
 */
final class TireFilterWidget extends WC_Widget
{
    /** @var TireRepository */
    private $repo;

    /**
     * Konstruktor.
     * @param TireRepository $repo
     */
    public function __construct(TireRepository $repo)
    {
        $this->repo = $repo;

        $this->widget_cssclass    = 'woocommerce widget_gath_tire_filter';
        $this->widget_description = 'Obrazac za filtriranje guma';
        $this->widget_id          = 'inti_gath_tire_filter';
        $this->widget_name        = 'Filter guma';
        $this->settings           = array(
            'title'              => array(
                'type'  => 'text',
                'std'   => 'Filter guma',
                'label' => 'Naslov',
            ),
            'text' => [
                'type' => 'textarea',
                'std' => '',
                'label' => 'Tekst ispod obrasca',
            ],
            'submit' => [
                'type' => 'text',
                'std' => 'Pronađi gume',
                'label' => 'Tekst na gumbu',
            ],
        );

        parent::__construct();

        add_action('wp_enqueue_scripts', [$this, 'wp_enqueue_scripts']);
    }

    public function wp_enqueue_scripts()
    {
        wp_enqueue_style('inti-gath-tire-filter', INTI_GATH_URL . '/css/tire-filter.css');
    }

    public function widget($args, $instance)
    {
        if (!is_product_category()) {
            return;
        }

        /** @var WP_Term $category */
        $category = get_queried_object();

        if (!get_term_meta($category->term_id, 'isTireCategory', true)) {
            return;
        }

        $this->widget_start($args, $instance);

        $properties = $this->repo->getProperties();
        ?>
        <form class="inti-tire-filter" method="get" action="<?= get_term_link(get_queried_object_id()); ?>">
            <?php
            foreach ($properties as $property) {
                echo $this->renderSelect(sprintf('%s:', $property->getName()), $property->getName(), $property->getValues(), 'SVE');
            }
            /*
            echo $this->renderSelect('Vrsta vozila:', 'vehicleType', $values->getVehicleTypes(), 'SVE VRSTE');
            echo $this->renderSelect('Sezona:', 'season', $values->getSeasons(), 'SVE GUME');
            echo $this->renderSelect('Brand:', 'brand', $values->getBrands(), 'SVI PROIZVOĐAČI');
            echo $this->renderSelect('Širina:', 'width', $values->getWidths(), 'SVE');
            echo $this->renderSelect('Visina:', 'height', $values->getHeights(), 'SVE');
            echo $this->renderSelect('Promjer:', 'diameter', $values->getDiameters(), 'SVE');
            */
            if (isset($instance['text']) && !empty($instance['text'])) {
                echo sprintf('<div class="inti-tire-filter-text">%s</div>', $instance['text']);
            }
            ?>

            <div class="inti-tire-filter-submit">
                <button type="submit"><?= ($instance['submit'] ?? '') ?: 'Pronađi gume'; ?></button>
            </div>
        </form>
        <?php
        $this->widget_end($args);
    }

    private function renderSelect(string $label, string $name, array $values, string $defaultValue = ''): string
    {
        $options[] = sprintf('<option value="">%s</option>', $defaultValue);

        foreach ($values as $value) {
            $options[] = sprintf(
                '<option value="%s"%s>%s</option>',
                $value,
                isset($_GET[$name]) && $_GET[$name] === ((string) $value) ? ' selected' : '',
                $value
            );
        }

        $label = sprintf('<label>%s</label>', $label);
        $select = sprintf('<div class="select-wrapper"><select name="%s">%s</select></div>', $name, implode('', $options));

        return sprintf('<div class="inti-tire-filter-%s">%s%s</div>', $name, $label, $select);
    }
}
